/* encrypt.c - encrypt the image
 *
 * Copyright (C) 2001  Jochen Voss.  */

static const  char  rcsid[] = "$Id: encrypt.c,v 1.2 2001/09/17 22:58:57 voss Rel $";

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>
#include <stdio.h>

#include <glib.h>
#include <jvrand.h>

#include "graphcrypt.h"


static  uchar *key_data = NULL;
static  int  key_no = -1;

void
generate_key (int key)
{
  int  i, w, h;

  w = page_width ();
  h = page_height ();
  g_assert (key >= 0);
  if (key == key_no)  return;

  key_no = key;
  jvrand_set_seed (key);
  g_free (key_data);
  key_data = g_new (uchar, w*h);
  for (i=0; i<w*h; ++i)  key_data [i] = jvrand_discrete (0, 4);
}

static void
put_pixquad (uchar *dest, int w, int i, int j, uchar val)
{
  int  ii = 2*i;
  int  jj = 2*j;
  int  ww = 2*w;
  
  switch (val) {
  case 0:
    /*  #.  *
     *  #.  */
    dest[ii+ww*jj] = 0;
    dest[ii+ww*jj+1] = 255;
    dest[ii+ww*(jj+1)] = 0;
    dest[ii+ww*(jj+1)+1] = 255;
    break;
  case 1:
    /*  ..  *
     *  ##  */
    dest[ii+ww*jj] = 0;
    dest[ii+ww*jj+1] = 0;
    dest[ii+ww*(jj+1)] = 255;
    dest[ii+ww*(jj+1)+1] = 255;
    break;
  case 2:
    /*  .#  *
     *  .#  */
    dest[ii+ww*jj] = 255;
    dest[ii+ww*jj+1] = 0;
    dest[ii+ww*(jj+1)] = 255;
    dest[ii+ww*(jj+1)+1] = 0;
    break;
  case 3:
    /*  ##  *
     *  ..  */
    dest[ii+ww*jj] = 255;
    dest[ii+ww*jj+1] = 255;
    dest[ii+ww*(jj+1)] = 0;
    dest[ii+ww*(jj+1)+1] = 0;
    break;
  default:
    g_assert_not_reached ();
  }
}

void
postscript_print_key (const char *filename)
{
  int  i, j, w, h;
  struct image  im;
  char  title [128];

  w = page_width ();
  h = page_height ();
  im.w = 2 * w;
  im.h = 2 * h;
  im.pitch = 2 * w;
  im.data = g_new (uchar, im.w*im.h);

  for (j=0; j<h; ++j) {
    for (i=0; i<w; ++i) {
      put_pixquad (im.data, w, i, j, key_data[i+w*j]);
    }
  }
  sprintf (title, "key %d-%d", key_no, DPI);
  postscript_print_image (filename, title, &im, 0, 0);
  g_free (im.data);
}

static uchar
crypt_pixel (uchar key, uchar val)
{
  uchar x;

  if (val < 32) {
    x = 2;
  } else if (val < 64+32) {
    x = 1;
  } else {
    x = 0;
  }
  return  (key+x)%4;
}

void
postscript_print_encrypted_image (const char *filename, const char *title,
				  struct image *im)
{
  int  i, j, kw, kh;
  int  xoffset, yoffset;
  struct image  crypt;
  char  title_str [128];

  kw = page_width ();
  kh = page_height ();
  g_assert (im->w <= kw && im->h <= kh);

  crypt.w = 2*im->w;
  crypt.h = 2*im->h;
  crypt.pitch = crypt.w;
  crypt.data = g_new (uchar, crypt.w*crypt.h);
  
  xoffset = jvrand_discrete (0, kw-im->w+1);
  yoffset = jvrand_discrete (0, kh-im->h+1);
  
  for (j=0; j<im->h; ++j) {
    for (i=0; i<im->w; ++i) {
      uchar  keyval = key_data[(i+xoffset)+kw*(j+yoffset)];
      uchar  imval = im->data[i+im->pitch*j];
      put_pixquad (crypt.data, im->w, i, j, crypt_pixel(keyval, imval));
    }
  }
  snprintf (title_str, 128, "%s encrypted for key %d-%d",
	    title, key_no, DPI);
  postscript_print_image (filename, title_str, &crypt, 2*xoffset, 2*yoffset);
  g_free (crypt.data);
}
