/* main.c - graphcrypt main file
 *
 * Copyright (C) 2001  Jochen Voss.  */

static const  char  rcsid[] = "$Id: main.c,v 1.1.1.1 2001/09/16 14:23:53 voss Rel $";

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include <errno.h>

#include <glib.h>
#include <jvrand.h>

#include "graphcrypt.h"


long int  key_id = 12345;


int
main (int argc, char **argv)
{
  struct option  long_options [] = {
    { "help", no_argument, 0, 'h' },
    { "key", required_argument, 0, 'k' },
    { "version", no_argument, 0, 'V' },
    { "write-key", optional_argument, 0, 'w' },
    { NULL, 0, NULL, 0}
  };
#define GC_SHORT_OPTIONS "hk:Vw::"
  int  help_flag = 0;
  int  version_flag = 0;
  char *key_file = NULL;
  int  error_flag = 0;

  do {
    int  c;
    int  ind;
    c = getopt_long (argc, argv, GC_SHORT_OPTIONS, long_options, &ind);
    if (c == -1)  break;
    switch (c) {
    case 'h':
      help_flag = 1;
      break;
    case 'k':
      {
	char *tail;
	long int  x;

	errno = 0;
	x = strtol (optarg, &tail, 0);
	if (errno == ERANGE && *tail == '\0') {
	  fprintf (stderr, "error: invalid number \"%s\"\n", optarg);
	  error_flag = 1;
	} else if (errno || tail == optarg || *tail != '\0') {
	  fprintf (stderr, "error: cannot convert \"%s\" into a number\n",
		   optarg);
	  error_flag = 1;
	} else if (x<0) {
	  fprintf (stderr, "error: the keyid must be positive\n");
	  error_flag = 1;
	} else {
	  key_id = x;
	}
      }
      break;
    case 'V':
      version_flag = 1;
      break;
    case 'w':
      g_free (key_file);
      if (optarg) {
	key_file = g_strdup (optarg);
      } else {
	key_file = g_strdup_printf ("key-%ld-%d.ps", key_id, DPI);
      }
      break;
    default:
      error_flag = 1;
    }
  } while (! error_flag);

  if (argc > optind+1) {
    fputs ("too many arguments\n", stderr);
    error_flag = 1;
  }
  
  if (version_flag) {
    puts (PACKAGE " " VERSION);
    puts ("Copyright 2001 Jochen Voss");
    puts ("\
GraphCrypt comes with NO WARRANTY, to the extent permitted by law.");
    puts ("\
You may redistribute copies of GraphCrypt under the terms of the GNU\n\
General Public License.  For more information about these matters, see\n\
the file named COPYING.");
    if (! error_flag)  exit (0);
  }
  if (error_flag || help_flag) {
    FILE *out = error_flag ? stderr : stdout;
    fprintf (out, "usage: graphcrypt [options] [image]\n\n");
    fputs ("The options are\n", out);
    fputs ("  -h, --help             show this message and exit\n", out);
    fputs ("  -k, --key ID           select the key to use\n", out);
    fputs ("  -V, --version          show the version number and exit\n", out);
    fprintf (out, "  -w, --write-key [FILE] "
	     "write key to FILE (default key-%ld-%d.ps)\n\n", key_id, DPI);
    fputs ("Please report bugs to <voss@mathematik.uni-kl.de>.\n", out);
    exit (error_flag);
  }

  jvrand_init ();
  generate_key (key_id);

  if (key_file) {
    printf ("writing key to file \"%s\" ...", key_file);
    fflush (NULL);
    postscript_print_key (key_file);
    puts (" done");
  }

  if (optind < argc) {
    char *infile, *tail, *outfile;
    infile = argv[optind];
    tail = strrchr (infile, '.');
    if (! tail)  tail = infile+strlen(infile);
    outfile = g_strdup_printf ("%.*s-%ld-%d.ps",
			       tail-infile, infile, key_id, DPI);
    handle_image (infile, outfile);
    g_free (outfile);
  }
  
  return  0;
}
